//----------------------------------------------------------------------------//
// COMP 239 Final Project                                                     //
// Adrian Ilie                                                                //
// Uses the Cal3D character animation library                                 //
// Copyright (C) 2001, 2002 Bruno 'Beosil' Heidelberger                       //
//----------------------------------------------------------------------------//

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

//----------------------------------------------------------------------------//
// Includes                                                                   //
//----------------------------------------------------------------------------//

#include "global.h"
#include "viewer.h"

#include "trackers.h"
#include "position.h"

//----------------------------------------------------------------------------//
// GLUT callback functions                                                    //
//----------------------------------------------------------------------------//

void displayFunc()
{
	int i;

	for (i=0; i<6; i++) Trackers[i].numReports=0;

	//tracking stuff
	tHiball->mainloop();
	tFastrack->mainloop();
	ComputeNewPosition(theViewer.m_calModel);

	theViewer.setModelPos(Trackers[2].Pos);

  // clear all the buffers
  glClearColor(0.0f, 0.0f, 0.3f, 0.0f);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	q_vec_type eye;
	q_vec_type lookat;
	static q_vec_type eye_in_hiball = { 0.0, 0.0, 0.0 };
	static q_vec_type lookat_in_hiball = { 0.0, -10.0, 0.0 };

	// transform from HiBall coordinates to room coordinates
	q_type hiballRot;
	CalQuaternion q = Trackers[0].Rot;
	for ( i = 0; i < 4; i++ )
		hiballRot[i] = q[i];

	q_xform( eye, hiballRot, eye_in_hiball );
	q_xform( lookat, hiballRot, lookat_in_hiball );

	CalVector Eye, LookAt, EyeR, LookAtR;
//	q.invert();
	Eye = CalVector(0,0,0);
	Eye *= q;
	LookAt = CalVector(0,10,0);
	LookAt *= q;
	lookat[1] = -lookat[1];
	//compute head position and orientation in world space=tracker space
	for ( i = 0; i < 3; i++ )
	{
		Eye[i] = eye[i]+Trackers[0].Pos[i];
		LookAt[i] = lookat[i]+Trackers[0].Pos[i];
	}

	//left eye
	EyeR = Eye + LeftEyeOffset;
	LookAtR = LookAt + LeftEyeOffset;
	if (defaultView) theViewer.setDefaultView();
	else theViewer.setView(EyeR, LookAtR);
	// render the scene
	theViewer.setViewport(0,0,640,480);
	theViewer.onRender();

	//right eye
	EyeR = Eye + RightEyeOffset;
	LookAtR = LookAt + RightEyeOffset;
	if (defaultView) theViewer.setDefaultView();
	else theViewer.setView(EyeR, LookAtR);
	// render the scene
	theViewer.setViewport(640,0,640,480);
	theViewer.onRender();

  // swap the front- and back-buffer
  glutSwapBuffers();
}

void exitFunc()
{
  // shut the viewer instance down
  theViewer.onShutdown();
}

void idleFunc()
{
  // redirect to the viewer instance
  theViewer.onIdle();
}

void keyboardFunc(unsigned char key, int x, int y)
{
	if ((key=='d')||(key=='D'))defaultView=!defaultView;
  	if ((key=='i')||(key=='I'))doIK=!doIK;
 // redirect the message to the viewer instance
  theViewer.onKey(key, x, theViewer.getHeight() - y - 1);
}

void motionFunc(int x, int y)
{
  // redirect the message to the viewer instance
  theViewer.onMouseMove(x, theViewer.getHeight() - y - 1);
}

void mouseFunc(int button, int state, int x, int y)
{
  // redirect the message to the viewer instance
  if(state == GLUT_DOWN)
  {
    theViewer.onMouseButtonDown(button, x, theViewer.getHeight() - y - 1);
  }
  else if(state == GLUT_UP)
  {
    theViewer.onMouseButtonUp(button, x, theViewer.getHeight() - y - 1);
  }
}

void reshapeFunc(int width, int height)
{
  // set the new width/height values
  theViewer.setDimension(width, height);
}

//----------------------------------------------------------------------------//
// Main entry point of the application                                        //
//----------------------------------------------------------------------------//

int main(int argc, char *argv[])
{
  // initialize the GLUT system
  glutInit(&argc, argv);

  // create our view instance
  if(!theViewer.onCreate(argc, argv))
  {
    std::cerr << "Creation of the viewer failed." << std::endl;
    return -1;
  }

  // register our own exit callback
  atexit(exitFunc);

	// connect to HiBalls
	tHiball = new vrpn_Tracker_Remote( hiballsPort );
	if ( tHiball == NULL )
	{
		fprintf( stderr, "Cannot create HiBall connection.\n" );
	}
	else
	{
		fprintf( stderr, "HiBalls connection established.\n" );
		tHiball->register_change_handler( NULL, (vrpn_TRACKERCHANGEHANDLER)hiballsHandler );
	}

	// connect to Fastrack
	tFastrack = new vrpn_Tracker_Remote( fastrackPort );
	if ( tFastrack == NULL )
	{
		fprintf( stderr, "Cannot create Fastrack connection." );
	}
	else
	{
		fprintf( stderr, "Fastrack connection established.\n" );
		tFastrack->register_change_handler( NULL, (vrpn_TRACKERCHANGEHANDLER)birdsHandler );
	}

	// connect to Python
	bPython = new vrpn_Button_Remote( pythonPort );
	if ( bPython == NULL )
	{
		fprintf( stderr, "Cannot create Python connection." );
	}
	else
	{
		fprintf( stderr, "Python connection established.\n" );
		bPython->register_change_handler( NULL, pythonHandler );
	}

  // set all GLUT modes
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
  glutInitWindowSize(theViewer.getWidth()*2, theViewer.getHeight());
  glutCreateWindow(theViewer.getCaption().c_str());
  if(theViewer.getFullscreen()) glutFullScreen();
  glutSetCursor(GLUT_CURSOR_NONE);

  // register all GLUT callback functions
  glutIdleFunc(idleFunc);
  glutMouseFunc(mouseFunc);
  glutMotionFunc(motionFunc);
  glutPassiveMotionFunc(motionFunc);
  glutReshapeFunc(reshapeFunc);
  glutDisplayFunc(displayFunc);
  glutKeyboardFunc(keyboardFunc);

  // initialize our viewer instance
  if(!theViewer.onInit())
  {
    std::cerr << "Initialization of the viewer failed." << std::endl;
    return -1;
  }

  modelName = theViewer.m_calModel.getSkeleton()->getBone(0)->getCoreBone()->getName();

  // run the GLUT message loop
  glutMainLoop();

  return 0;
}

//----------------------------------------------------------------------------//
