#ifndef POSITION_H
#define POSITION_H

#include "global.h"
#include "trackers.h"

static CalVector Tracker2InModel = CalVector(0,0,130);
static std::string modelName="Cally";
static bool doIK=false;
static bool defaultView=true;
static CalVector LeftEyeOffset = CalVector(-3, 22, -3);
static CalVector RightEyeOffset = CalVector(3, 22, -3);

void SetBoneRotation(CalModel &model, CalBone *bone, CalQuaternion rotation)
{
	int boneID, parentID;
	CalBone *parentBone;

	parentID = bone->getCoreBone()->getParentId();
	parentBone = model.getSkeleton()->getBone( parentID );

//	bone->setCoreState();

	CalQuaternion parentRot = parentBone->getRotationAbsolute();//same rotation
	CalQuaternion boneRot = bone->getRotation();//same rotation
	parentRot.invert();
	parentRot = parentRot * rotation;

	//the effect of this is to set RotationAbsolute to the value from the tracker (rotated)
	bone->setRotation(parentRot);
    bone->lockState();
    bone->calculateState();
	boneRot = bone->getRotationAbsolute();
}

void PointAt( CalModel &model, std::string &boneName, CalVector targetPos )
{

	int childID, parentID;
	CalBone *childBone;
	CalBone *parentBone;

	childID = model.getCoreModel()->getBoneId( boneName );
	childBone = model.getSkeleton()->getBone( childID );

	parentID = childBone->getCoreBone()->getParentId();
	parentBone = model.getSkeleton()->getBone( parentID );


	parentBone->setCoreState();
	childBone->setCoreState();

   //This is the direction the child bone is pointing now, in the parent
   //bone's coordinate system
   CalVector currentDirection = childBone->getTranslation();

   //Convert target position into the parent bone's coordinate system
   targetPos -= parentBone->getTranslationAbsolute();
   CalQuaternion tempRot = parentBone->getRotationAbsolute();
   tempRot.invert();
   targetPos *= tempRot;

   //Normalize both vectors for shortest arc calculation
   currentDirection.normalize();
   targetPos.normalize();

   //Get the current rotation that results in the child bone's current
   //direction
   CalQuaternion parentRot = parentBone->getRotation();

   //Multiply it by the shortest arc quaternion from the current direction
   //to the desired direction
   //Note: This shorestArc function is in the CalQuaternion class, so
   //      you might have to make it CalQuaternion::shortestArc(...)
   ///     instead.
   parentRot *= shortestArc( currentDirection, targetPos );


   //Set the parent bone and recalculate its children's states
   parentBone->setRotation( parentRot );
//   SetBoneRotation(model,parentBone,parentRot);
   parentBone->calculateState();

}

void LookAt( CalModel &model )
{
	CalQuaternion q = Trackers[0].Rot;//get the tracker rotation
	CalVector target = CalVector(0,1,0);//lookat in tracker coords
	target *= q;//transform
//	q = CalQuaternion(-1,0,0,1);
//	target *= q;//transform

//	target += Trackers[1].Pos;//shift into position
//	target -= Trackers[2].Pos - Tracker2InModel;

	int neckBoneID = model.getCoreModel()->getBoneId( modelName+" Head" );
	//Note that the target vector is relative to your model's base coordinate system - NOT relative to the head

	//Get the neck bone
	CalBone* neckBone = model.getSkeleton()->getBone(neckBoneID);

	//Make the target vector relative to the head
//	target = target - neckBone->getTranslationAbsolute();

	//We can't look at the head's position, so return
	if (target == CalVector(0,0,0)) return;

	//Normalize the direction (required for shortestArc calculation)
	target.normalize();

	//This is the head's original direction
	CalVector original (0, -1, 0);
	CalQuaternion headRot = shortestArc( original, target );
	headRot.conjugate();

	//Mess with the quaternion (I'm not sure why we have to do this)
	CalQuaternion tempRot = headRot;
	headRot.x = tempRot.z;
	headRot.z = tempRot.x;
	headRot.y = tempRot.y;

	//Set the rotation and recalculate the state
	neckBone->setRotation( headRot );
	neckBone->calculateState();
}

int InverseKinematics(CalModel &model, std::string &startBoneName, std::string &endBoneName, CalVector target, float tolerance, int numIter)
{
	int i=0;
	int startID, endID, parentID, currentID, lengthID;
	CalBone *startBone, *endBone, *parentBone, *currentBone, *lengthBone;
	float distance;
	CalVector Distance, currentDirection, targetPos;
	CalQuaternion tempRot, parentRot;

	startID = model.getCoreModel()->getBoneId( startBoneName );
	startBone = model.getSkeleton()->getBone( startID );
	endID = model.getCoreModel()->getBoneId( endBoneName );
	endBone = model.getSkeleton()->getBone( endID );
	//check if we received valid bone names
	if ((!startBone)||(!endBone))return -1;//invalid input, cannot find bones
	//check if the bones form a chain
	currentBone = endBone;//start with the end of the chain
	do
	{//go backwards to make sure we can reach the start
		currentID = currentBone->getCoreBone()->getParentId();
		currentBone = model.getSkeleton()->getBone( currentID );
	}
	while ((currentID!=startID)&&(currentID!=-1));//look for the start, or the root
	if (currentID==-1)return -2;//invalid input, bones do not form a chain
	//done checking, time for work
	do
	{
		//initialize with the parent of the last segment in the chain
//		currentID = endBone->getCoreBone()->getParentId();
//		currentBone = model.getSkeleton()->getBone( currentID );
		currentDirection = CalVector(0,0,0);
		currentBone = endBone;
		currentID = endID;
		do
		{//start going backwards
			parentID = currentBone->getCoreBone()->getParentId();
			parentBone = model.getSkeleton()->getBone( parentID );
			//this is the direction the current subchain is pointing now, in the parent bone's coordinate system
			currentDirection = endBone->getTranslationAbsolute() - parentBone->getTranslationAbsolute();
//			CalVector t=currentBone->getTranslationAbsolute()-parentBone->getTranslationAbsolute();
//			t=currentBone->getTranslation();
//			t*=parentBone->getRotationAbsolute();
//			currentDirection += t;
//			Distance = endBone->getTranslationAbsolute() - parentBone->getTranslationAbsolute();

			currentDirection = CalVector(0,0,0);
			lengthBone = endBone;
			lengthID = endID;
			if (lengthID != currentID) do
			{
				currentDirection += lengthBone->getTranslation();
				lengthID = lengthBone->getCoreBone()->getParentId();
				lengthBone = model.getSkeleton()->getBone( lengthID );
			}
			while (lengthID != currentID);
			else currentDirection = currentBone->getTranslation();


			//convert target position into the parent bone's coordinate system
			targetPos = target - parentBone->getTranslationAbsolute();
			tempRot = parentBone->getRotationAbsolute();
			tempRot.invert();
			targetPos *= tempRot;
//			currentDirection *= tempRot;
			//normalize both vectors for shortest arc calculation
			currentDirection.normalize();
			targetPos.normalize();
			//get the current rotation that results in the child bone's current direction
			parentRot = parentBone->getRotation();
			//multiply it by the shortest arc quaternion from the current direction to the desired direction
			parentRot *= shortestArc( currentDirection, targetPos );
			//set the parent bone rotation and recalculate its children's states
			parentBone->setRotation( parentRot );
			parentBone->calculateState();
			//advance backwards in the chain
			currentBone = parentBone;
			currentID = parentID;
		}
		while (parentID!=startID);//look for the start
		i++;//next iteration
		//see if we reached the goal
		Distance = endBone->getTranslationAbsolute() - target;
		distance = Distance.length();
	}
	while ((i<numIter)&&(distance>tolerance));
	//done, return appropriate status value
	if (i==numIter)return -3;//goal not reached within given tolerance
	else return 0;//goal reached within given tolerance
}

void SetBoneRotation(CalModel &model, std::string &boneName, int trackerNo)//, CalQuaternion &offset)
{
	int boneID, parentID;
	CalBone *bone, *parentBone;

	boneID = model.getCoreModel()->getBoneId( boneName );
	bone = model.getSkeleton()->getBone( boneID );
	parentID = bone->getCoreBone()->getParentId();
	parentBone = model.getSkeleton()->getBone( parentID );

	bone->setCoreState();

	CalQuaternion parentRot = parentBone->getRotationAbsolute();//same rotation
	CalQuaternion boneRot = bone->getRotation();//same rotation
	parentRot.invert();
	parentRot = parentRot * Trackers[trackerNo].Rot;

	//Mess with the quaternion (I'm not sure why we have to do this)
	//parentRot.invert();
	CalQuaternion tempRot = parentRot;
	parentRot.x = tempRot.z;
	parentRot.z = tempRot.y;
	parentRot.y = tempRot.x;

	//the effect of this is to set RotationAbsolute to the value from the tracker (rotated)
	bone->setRotation(parentRot);
    bone->lockState();
    bone->calculateState();
	boneRot = bone->getRotationAbsolute();
}

void ComputeNewPosition(CalModel &model)
{
	// clear the skeleton state
	model.getSkeleton()->clearState();
	int boneID;
	CalBone *bone;
	CalVector temp;
	boneID = model.getCoreModel()->getBoneId( modelName );
	bone = model.getSkeleton()->getBone( boneID );
	bone->setCoreStateRecursive();

	if(doIK)
	{
		//Inverse Kinematics stuff goes here
		CalVector lookAt = CalVector( 0.0, -20.0, 20.0 );
		lookAt *= Trackers[0].Rot;
		lookAt = lookAt + Trackers[0].Pos-Trackers[2].Pos+Tracker2InModel;
		temp[0]=-lookAt[0];
		temp[1]=-lookAt[1];
		temp[2]=lookAt[2];
		InverseKinematics(model,modelName+" Spine1",modelName+" Head",temp,10,5);
	}

	CalVector target;

	//left hand
	target = Trackers[3].Pos-Trackers[2].Pos+Tracker2InModel;
	temp[0]=-target[0];
	temp[1]=-target[1];
	temp[2]=target[2];
	PointAt(model, modelName+" L Forearm", temp);
	target = Trackers[1].Pos-Trackers[2].Pos+Tracker2InModel;
	temp[0]=-target[0];
	temp[1]=-target[1];
	temp[2]=target[2];
	PointAt(model, modelName+" L Hand", temp);
//	PointAt(model, modelName+" L Finger0", temp);
//	PointAt(model, modelName+" L Finger01", temp);
//	PointAt(model, modelName+" L Finger02", temp);
//	SetBoneRotation(model, modelName+" L Hand", 1);//right hand, hiball 1=tracker 1
	//right hand
	target = Trackers[5].Pos-Trackers[2].Pos+Tracker2InModel;
	temp[0]=-target[0];
	temp[1]=-target[1];
	temp[2]=target[2];
	PointAt(model, modelName+" R Forearm", temp);
	target = Trackers[4].Pos-Trackers[2].Pos+Tracker2InModel;
	temp[0]=-target[0];
	temp[1]=-target[1];
	temp[2]=target[2];
	PointAt(model, modelName+" R Hand", temp);
//	PointAt(model, modelName+" R Finger0", temp);
//	PointAt(model, modelName+" R Finger01", temp);
//	PointAt(model, modelName+" R Finger02", temp);
//	SetBoneRotation(model, modelName+" R Hand", 4);//right hand, bird 2=tracker 1
		//torso
//		SetBoneRotation(model, modelName+" Pelvis", 2);//right hand, hiball 1=tracker 1
//		target = Trackers[0].Pos-Trackers[2].Pos+Tracker2InModel;
//		temp[0]=-target[0];
//		temp[1]=-target[1];
//		temp[2]=target[2];
		//PointAt(model, modelName+" Spine2",temp);
		//model.getSkeleton()->getBone(model.getCoreModel()->getBoneId(modelName+" Pelvis"))->setRotation(Trackers[2].Rot);
	//head - turn it the way it should be
	LookAt(model);
	// lock the skeleton state
	model.getSkeleton()->lockState();
	// let the skeleton calculate its final state
	model.getSkeleton()->calculateState();
	model.getPhysique()->update();
}

#endif