#ifndef TRACKERS_H
#define TRACKERS_H

#include "quat.h"
#include "vrpn_Tracker.h"
#include "vrpn_Button.h"
#include "cal3d/vector.h"
#include "cal3d/quaternion.h"

//----------------------------------------------------------------------------//
// Tracker stuff                                                              //
//----------------------------------------------------------------------------//
struct Tracker
{
	CalVector Pos;
	CalQuaternion Rot;
	int numReports;
} Trackers[6];//2 Hiballs and 4 Birds

static vrpn_Tracker_Remote *tHiball, *tFastrack;
static vrpn_Button_Remote *bPython;
static const char hiballsPort[]="Tracker0@tracker1-cs.cs.unc.edu";
static const char fastrackPort[]="Fastrak0@ioglab3.cs.unc.edu";
static const char pythonPort[] = "Button0@tracker1-cs.cs.unc.edu";
//static CalVector SensorInHiball = CalVector(268.7, 940.2, 200.3);
static CalVector SensorInHiball = CalVector(397.503, 915.811, 219.504);
static CalVector Reference;//coords of glove bird (tracker 4=bird 2) in sensor coord system

//----------------------------------------------------------------------------//
// VRPN callback functions                                                    //
//----------------------------------------------------------------------------//

static void hiballsHandler( void *userdata, const vrpn_TRACKERCB hb )
{
	int i;
	for(i=0; i<2; i++)
	{
		if ( hb.sensor == i )
		{
			Trackers[i].Pos[0] = hb.pos[0] * 100.0;	// convert m to cm
			Trackers[i].Pos[1] = hb.pos[1] * 100.0;
			Trackers[i].Pos[2] = hb.pos[2] * 100.0;
			Trackers[i].Rot[Q_X] = hb.quat[Q_X];
			Trackers[i].Rot[Q_Y] = hb.quat[Q_Y];
			Trackers[i].Rot[Q_Z] = hb.quat[Q_Z];
			Trackers[i].Rot[Q_W] = hb.quat[Q_W];
//			printf( "Hiball %d:%f %f %f\n", i, (double)hb.pos[0], (double)hb.pos[1], 
//								    (double)hb.pos[2] );
			Trackers[i].numReports++;
		}
	}
}

static void birdsHandler( void *userdata, const vrpn_TRACKERCB hb )
{
	static CalVector Pos;
	int i;
	for(i=0; i<4; i++)
	{
		if ( hb.sensor == i )
		{
			Pos[0] = hb.pos[0] * 100.0;	// convert to m to cm
			Pos[1] = hb.pos[1] * 100.0;
			Pos[2] = hb.pos[2] * 100.0;

			Trackers[i+2].Pos[0] = SensorInHiball[0] + Pos[1];
			Trackers[i+2].Pos[1] = SensorInHiball[1] - Pos[0];
			Trackers[i+2].Pos[2] = SensorInHiball[2] + Pos[2];
//			Trackers[i+2].Pos = Pos;

			Trackers[i+2].Rot[Q_X] = hb.quat[Q_X];
			Trackers[i+2].Rot[Q_Y] = hb.quat[Q_Y];
			Trackers[i+2].Rot[Q_Z] = hb.quat[Q_Z];
			Trackers[i+2].Rot[Q_W] = hb.quat[Q_W];
//			printf( "Bird %d:%f %f %f\n",i, (double)hb.pos[0], (double)hb.pos[1], 
//								    (double)hb.pos[2] );
			Trackers[i+2].numReports++;
		}
	}
	if ( hb.sensor == 2 )
	{
		Reference[0] = hb.pos[0] * 100.0;	// convert to m to cm
		Reference[1] = hb.pos[1] * 100.0;
		Reference[2] = hb.pos[2] * 100.0;
	}
}

static void pythonHandler( void *userdata, const vrpn_BUTTONCB b )
{
	// b.button = 0, 1 or 2
	// b.state = 0 (released), 1 (pressed)
	printf("Button %d state %d\n",b.button,b.state);
	if ((b.button == 1)&&(b.state == 1))
	{
		printf("Calibration.\n");
		SensorInHiball[0] = Trackers[1].Pos[0] - Reference[1];
		SensorInHiball[1] = Trackers[1].Pos[1] + Reference[0];
		SensorInHiball[2] = Trackers[1].Pos[2] - Reference[2];
	}
}

#endif